@@ -34,13 +34,14 @@
 typedef struct
 {
     char * name;
+    char * path;	/* filesystem "safe" version of name */
     size_t len;
     unsigned char * data;
     struct date dt;
+    char * mime_type;
 } File;
 
 extern void file_write (File *file, const char* directory);
-extern void file_add_attr (File* file, const char* directory, Attr* attr);
+extern void file_add_attr (File* file, Attr* attr);
 extern void file_free (File *file);
-
 #endif /* FILE_H */
